#import "DirectoryDataSource.h"
#import "DirectoryItem.h"
#import "../Utils/Utils.h"


@implementation DirectoryDataSource

@synthesize directoryBrowserView = mDirectoryBrowserView;

-(id)init
{
	DEBUG_LOG(@"DirectoryBrowserDataSource - init");

	self = [super init];
	if (self != nil)
	{
		// create the root directories. We want to be able to easily access
		// the home directory, and the mounted volumes.
		mRootDirectories = [[NSMutableArray alloc] init];
		[mRootDirectories addObject:[DirectoryItem itemWithPath:NSHomeDirectory()]];
		[mRootDirectories addObject:[DirectoryItem itemWithPath:@"/Volumes/"]];

		// update the view
		[mDirectoryBrowserView reloadData];
	}
	return self;
}

-(void)dealloc
{
	[mRootDirectories removeAllObjects];
	[mRootDirectories release];
	[super dealloc];
}

/**
	This method return the nth child of an item. If the item is nil, it's the
	nth root item.
*/
-(id)outlineView:(NSOutlineView *)outlineView child:(NSInteger)index ofItem:(id)item
{
	if (item == nil)
	{
		if (mRootDirectories != nil)
			return [mRootDirectories objectAtIndex:index];
		else
			return nil;
	}
	else
		return [[item children] objectAtIndex:index];
}

/**
	Returns YES if an item can be expanded (e.g. it has child(ren))
*/
-(BOOL)outlineView:(NSOutlineView *)outlineView isItemExpandable:(id)item
{
	if (item == nil || [item canExpand] != NO)
		return YES;
	return NO;
}

/**
	Returns the number of children of the given item. If item is nil, return the
	number of root items
*/
-(NSInteger)outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item
{
	if (item == nil)
	{
		if (mRootDirectories != nil)
			return [mRootDirectories count];
		else
			return 0;
	}
	return [item numChildren];
}

/**
	Return the value for the given item and column. We use the column's
	identifier and use it as a key to get the corresponding value in the item.
	This way, we can add as many column as we want without touching this method.
*/
-(id)outlineView:(NSOutlineView *)outlineView objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item
{
	if (item == nil)
		return @"/";

	// NOTE: we're only using 1 column here, so don't bother
	//		 We can however handle as man column as we want by using the key
	//		 [tableColumn identifier] This way, if we add a column with an
	//		 identifier named "foo", we simply have to add a "foo" property
	//		 in the DirectoryBrowserItem, and it will display itself.
	return [item valueForKey:@"name"];
}

@end
