#import "DirectoryItem.h"
#import "../Utils/FileUtils.h"
#import "../Utils/SimpleProfiler.h"


@implementation DirectoryItem

@synthesize name		= mName;
@synthesize path		= mPath;
@synthesize numImages	= mNumImages;


/**
	Utility method to create an autoreleased directory browser item.
*/
+(DirectoryItem *)itemWithPath:(NSString *)path
{
	return [[[DirectoryItem alloc] initWithPath:path] autorelease];
}

/**
	Initialize a directory browser item with a path.
*/
-(DirectoryItem *)initWithPath:(NSString *)path
{
	self = [super init];
	if (self != nil)
	{
		mPath			= [path retain];
		mName			= [[mPath lastPathComponent] retain];
		mChildren		= nil;
		mNumImages		= 0;
		mCanExpand		= NO;
		mNumChildren	= 0;
		
		PROFILING_START(@"DirectoryItem - initWithPath");
		
		// get the content of the directory
		NSFileManager * fileManager = [NSFileManager defaultManager];
		NSURL * url = [NSURL fileURLWithPath:mPath];
		mCachedContent = [[fileManager contentsOfDirectoryAtURL:url
									   includingPropertiesForKeys:nil
									   options:NSDirectoryEnumerationSkipsHiddenFiles
									   error:nil] retain];

		// parse the content, count the number of images and directories.
		for (NSURL * item in mCachedContent)
		{
			if (CFURLHasDirectoryPath((CFURLRef)item))
			{
				++mNumChildren;
				mCanExpand = YES;
			}
			else if ([FileUtils isImage:[item path]] == YES)
			{
				++mNumImages;
			}
		}
		
		PROFILING_STOP();
	}
	return self;
}

-(void)dealloc
{
	[mPath release];
	[mName release];
	[mChildren release];
	[super dealloc];
}

-(BOOL)canExpand
{
	return mCanExpand;
}

-(int)numChildren
{
	return mNumChildren;
}

-(NSMutableArray *)children
{
	// the first time children is accessed, initliaze its content
	if (mChildren == nil)
	{
		mChildren = [[NSMutableArray alloc] initWithCapacity:mNumChildren];

		PROFILING_START(@"DirectoryItem - children");

		// parse the content, count the number of images and directories.
		for (NSURL * item in mCachedContent)
		{
			if (CFURLHasDirectoryPath((CFURLRef)item))
			{
				// create a child item and store it
				DirectoryItem * child = [[DirectoryItem alloc] initWithPath:[item path]];
				[mChildren addObject:child];
				[child release];
			}
		}

		PROFILING_STOP();
	}
	return mChildren;
}

@end
