#import <Quartz/Quartz.h>
#import "ImageDataSource.h"
#import "ImageItem.h"
#import "../ImageBrowserView/ImageBrowser.h"
#import "../Utils/FileUtils.h"
#import "../Utils/Utils.h"


@implementation ImageDataSource

//@synthesize imageCount;
@synthesize imageBrowser = mImageBrowser;

-(id)init
{
	DEBUG_LOG(@"ImageBrowserDataSource - init");

	self = [super init];
	if (self != nil)
	{
		mCurrentDirectory = nil;
		mImages = [[NSMutableArray alloc] init];
		mImportedImages = [[NSMutableArray alloc] init];
	}
	return self;
}

-(void)dealloc
{
	[mImages release];
	[mImportedImages release];
	[super dealloc];
}

-(NSString *)currentDirectory
{
	return mCurrentDirectory;
}

-(void)setCurrentDirectory:(NSString *)path
{
	DEBUG_LOG(@"ImageDataSource - setCurrentDirectory:%@", path);

	if (path != mCurrentDirectory && path != nil)
	{
		// copy the new path
		[mCurrentDirectory release];
		mCurrentDirectory = [path copy];

		// reset the selection
		[mImages removeAllObjects];

		// create a new thread to load the images and start it
		[NSThread detachNewThreadSelector:@selector(addImagesInPath:)
				  toTarget:self
				  withObject:mCurrentDirectory];
	}
}

-(BOOL)imageCount
{
	return [mImages count] != 0 ? YES : NO;
}

-(void)setImageCount:(BOOL)imageCount
{
}

-(void)addImagesInPath:(NSString *)path
{
	NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];

	// get the content of the directory
	NSFileManager * fileManager = [[NSFileManager alloc] init];
	NSArray * content = [fileManager contentsOfDirectoryAtPath:path error:nil];

	// for each item, create an image object and add to the mImportedImages array
	for (NSString * p in content)
	{
		// ignore files beginning with '.'
		if ([[p lastPathComponent] characterAtIndex:0] == '.')
			continue;

		// complete file path
		NSString * file = [path stringByAppendingPathComponent:p];

		// ignore non-images files
		if ([FileUtils isImage:file] == NO)
			continue;

		ImageItem * image = [[ImageItem alloc] init];
		[image setPath:file];
		[image setTitle:[file lastPathComponent]];
		[mImportedImages addObject:image];
		[image release];
	}

	// when loading is done, update the data source in the main thread
	[self performSelectorOnMainThread:@selector(updateDatasource)
		  withObject:nil
		  waitUntilDone:NO];
	
	// cleanup
	[fileManager release];
	[pool release];
}

-(void)updateDatasource
{
	[mImages addObjectsFromArray:mImportedImages];
	[mImportedImages removeAllObjects];
	[mImageBrowser reloadData];
	[self setImageCount:[mImages count] == 0 ? YES : NO];
}

-(NSUInteger)numberOfItemsInImageBrowser:(IKImageBrowserView *)view
{
    return [mImages count];
}

-(id)imageBrowser:(IKImageBrowserView *)view itemAtIndex:(NSUInteger)index
{
    return [mImages objectAtIndex:index];
}

-(int)indexOfImage:(NSString *)image
{
	int index = 0;
	for (ImageItem * item in mImages)
	{
		if ([[item path] compare:image] == NSOrderedSame)
			return index;
		++index;
	}
	return -1;
}

-(void)imageBrowser:(IKImageBrowserView *)aBrowser removeItemsAtIndexes:(NSIndexSet *)indexes
{
	// remove the images from the list
	[mImages removeObjectsAtIndexes:indexes];
	
	// notify the view that it needs to reload its data
	// NOTE: I use reloadDataAndKeepSelection to select the next available image
	//		 automatically. A side effect is that you can use this in
	//		 fullscreen, it won't leave fullscreen and automatically select the
	//		 next image.
	[mImageBrowser reloadDataAndKeepSelection];
}

-(NSArray *)imagesAtIndexes:(NSIndexSet *)indexes
{
	NSMutableArray * images = [NSMutableArray arrayWithCapacity:[indexes count]];
	NSUInteger index = [indexes firstIndex];
	while (index != NSNotFound)
	{
		[images addObject:[self imageBrowser:nil itemAtIndex:index]];
		index = [indexes indexGreaterThanIndex:index];
	}	
	return images;
}

@end
