#import "DirectoryBrowserDelegate.h"
#import "DirectoryBrowserCell.h"
#import "../DataSources/DirectoryItem.h"


@implementation DirectoryBrowserDelegate

@synthesize currentDirectory = mCurrentDirectory;

-(id)init
{
	self = [super init];
	if (self != nil)
	{
		mCurrentDirectory = nil;
	}
	return self;
}

-(void)dealloc
{
	if (mCurrentDirectory != nil)
	{
		[mCurrentDirectory release];
		mCurrentDirectory = nil;
	}
	[super dealloc];	
}

/**
	This method is not part of the delegate protocol, but it's still being
	called each time the selection changes, so we can safely use it.
*/
-(void)outlineViewSelectionDidChange:(NSNotification *)notification
{
	NSOutlineView * outlineView = [notification object];
	NSInteger selectedRow = [outlineView selectedRow];
	DirectoryItem * item = [outlineView itemAtRow:selectedRow];
	[self setCurrentDirectory:[item path]];
}

/**
	This method is part of the delegate protocol and is used to provide a custom
	cell to represent an item.
*/
-(NSCell *)outlineView:(NSOutlineView *)outlineView
		   dataCellForTableColumn:(NSTableColumn *)tableColumn
		   item:(id)item
{
	// we don't want to return a unique cell for the whole row, so when tableColumn
	// is nil, return nil. This method will then be called again, this time with the
	// different columns
	if (tableColumn == nil)
		return nil;
	
	// NOTE: in this application, we only want 1 column, so don't bother with
	//		 tableColumn. If you want to use multiple column, you could use
	//		 [tableColumn identifier] to identify your columns in the interface
	//		 builder, and check it here to return the correct cell.
	return [DirectoryBrowserCell cellWithItem:(DirectoryItem *)item];
}

@end
