#import <Cocoa/Cocoa.h>
#import <Quartz/Quartz.h>


@class ImageDataSource;


/**
	This class is used to display a single image in a bigger view. It also allow
	automatic playing of animated gifs, fullscreen browsing, etc.
*/
@interface ImageView
	: IKImageView
{

@private

	NSURL *				mImageURL;
	int					mImageIndex;
	ImageDataSource *	mDataSource;

}

@property (assign) IBOutlet ImageDataSource * dataSource;

-(void)awakeFromNib;
-(void)dealloc;

// used to bind selected image to another view (when in fullscreen, the image
// view becomes first responder and allow to browse images, so we need to let
// other views be notified. This is done through binding with "currentImage")
-(NSString *)currentImage;
-(void)setCurrentImage:(NSString *)image;

// fullscreen handling
-(void)setFullscreen:(BOOL)fullscreen;
-(BOOL)fullscreen;
-(void)leaveFullscreen;
-(void)enterFullscreen;

// utilities
-(void)setImageWithURL:(NSURL *)url force:(BOOL)force;
-(void)reloadImage;
-(void)changeImage:(int)delta loop:(BOOL)loop;

// used by the slideshow timer
-(void)selectNextImage:(NSTimer *) timer;

// UI methods
-(BOOL)acceptsFirstResponder;
-(void)keyDown:(NSEvent *)theEvent;
-(void)mouseDown:(NSEvent *)theEvent;
-(void)otherMouseDown:(NSEvent *)theEvent;
-(void)scrollWheel:(NSEvent *)theEvent;

@end
