#import "ImageViewerAppDelegate.h"
#import "DirectoryView/DirectoryBrowserDelegate.h"
#import "DataSources/ImageDataSource.h"
#import "ImageBrowserView/ImageBrowser.h"
#import "ImageBrowserView/ImageBrowserDelegate.h"
#import "ImageView/ImageView.h"
#import "Preferences/PreferencesWindowController.h"
#import "Preferences/Preferences.h"
#import "Utils/FileUtils.h"
#import "Utils/SimpleProfiler.h"
#import "Utils/SlideShow.h"
#import "Utils/Utils.h"


@implementation ImageViewerAppDelegate


-(void)applicationDidFinishLaunching:(NSNotification *)aNotification
{			 
	DEBUG_LOG(@"ImageViewerAppDelegate - applicationDidFinishLaunching");

	// bind the currently selected directory of the directory browser to the
	// image browser view. This way, each time we select another directory,
	// the image browser view will update itself
	[Utils bind:mImageDataSource
		   keyPath:@"currentDirectory"
		   to:mDirectoryBrowserDelegate
		   keyPath:@"currentDirectory"
		   continuous:YES
		   twoWay:NO];

	// bind the image view and the image browser delegate. We do that to avoid
	// having direct reference of one class in the other and vice versa.
	// First, we need to bind the selected image, since the image view can
	// browse image in fullscreen, and the image browser needs to keep
	// synchronized.
	[Utils bind:mImageView
		   keyPath:@"currentImage"
		   to:mImageBrowserDelegate
		   keyPath:@"selectedImage"
		   continuous:YES
		   twoWay:YES];

	// we also need to bind the fullscreen : the image browser can request
	// fullscreen, but it is the image view which is responsible for this. And
	// when the image view leaves fullscreen, it needs to notify the image
	// browser
	[Utils bind:mImageView
		   keyPath:@"fullscreen"
		   to:mImageBrowserDelegate
		   keyPath:@"fullscreen"
		   continuous:YES
		   twoWay:YES];
	
	// NOTE: since Lion, sometime the views are scrolled to some strange values,
	//		 giving the impression that the view is empty. So we manually scroll
	//		 them once to the origin
	[mDirectoryBrowser scrollPoint:NSMakePoint(0.0f, [mDirectoryBrowser frame].size.height)];
	[mImageBrowser scrollPoint:NSMakePoint(0.0f, [mImageView frame].size.height)];

	// preferences
	Preferences * preferences = [Preferences instance];
	[preferences bind:mImageView
				 key:@"backgroundColor"
				 to:@"backgroundColor"
				 withUnarchiver:YES];
	[preferences bind:mImageBrowser
				 key:@"zoomValue"
				 to:@"thumbnailSize"
				 withUnarchiver:NO];
	[preferences bind:mImageBrowser
				 key:@"showTitles"
				 to:@"showTitles"
				 withUnarchiver:NO];
	[preferences bind:mImageBrowser
				 key:@"backgroundColor"
				 to:@"backgroundColor"
				 withUnarchiver:YES];
}

-(void)applicationWillTerminate:(NSNotification *)notification
{
	DEBUG_LOG(@"ImageViewerAppDelegate - applicationWillTerminate");

	// do a little cleanup
	[[SimpleProfiler instance] log];
	[SimpleProfiler destroyInstance];

	// unbind
	[mImageView unbind:@"fullscreen"];
	[mImageBrowserDelegate unbind:@"fullscreen"];
	[mImageDataSource unbind:@"currentDirectory"];
	[mImageView unbind:@"currentImage"];
	[mImageBrowserDelegate unbind:@"selectedImage"];

	// unbind preferences
	[mImageView unbind:@"backgroundColor"];
	[mImageBrowser unbind:@"zoomValue"];
	[mImageBrowser unbind:@"showTitles"];
	[mImageBrowser unbind:@"backgroundColor"];

	// ensure the image view is not in fullscreen
	[mImageView leaveFullscreen];

	// cleanup
	[Preferences destroy];
	[SlideShow stopSlideShow];
}

-(IBAction)showPreferences:(id)sender
{
	[[PreferencesWindowController instance] runModal];
}

-(IBAction)deleteItems:(id)sender
{
	DEBUG_LOG(@"ImageViewerAppDelegate - deleteItems");

	if ([mWindow firstResponder] == mImageBrowser)
		[mImageBrowser deleteSelectedImages];
}

-(BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)sender
{
	return YES;
}

@end
