#import "FileUtils.h"
#import "Utils.h"
#import "SimpleProfiler.h"
#import "../Preferences/Preferences.h"


@implementation FileUtils


static FileUtils * instance = nil;


-(void)awakeFromNib
{
	instance = self;
}

-(void)dealloc
{
	instance = nil;
	[super dealloc];
}

+(FileUtils *)instance
{
	return instance;
}

+(BOOL)isImage:(NSString *)path
{
	PROFILING_START(@"FileUtils - isImage");

	NSString * extension = [[path pathExtension] lowercaseString];
	if ([extension isEqualToString:@"jpg"] == YES ||
		[extension isEqualToString:@"jpeg"] == YES ||
		[extension isEqualToString:@"gif"] == YES ||
		[extension isEqualToString:@"png"] == YES ||
		[extension isEqualToString:@"psd"] == YES ||
		[extension isEqualToString:@"tiff"] == YES ||
		[extension isEqualToString:@"tif"] == YES ||
		[extension isEqualToString:@"dng"] == YES ||
		[extension isEqualToString:@"cr2"] == YES ||
		[extension isEqualToString:@"raw"] == YES ||
		[extension isEqualToString:@"pdf"] == YES)
	{
		PROFILING_STOP();
		return YES;
	}

	PROFILING_STOP();
	return NO;
}

+(BOOL)isGIF:(NSString *)path
{
	NSString * extension = [[path pathExtension] lowercaseString];
	return [extension isEqualToString:@"gif"];
}

+(void)removeFileAtPath:(NSString *)path
{
	// check the preferences to see if we need to use the recycled bin, or
	// permanently delete files
	BOOL permanently = [[Preferences instance] boolForKey:@"permanentlyDeleteFiles"];

	DEBUG_LOG(@"FileUtils - removeFileAtPath:%@ permanently:%@",
			  path,
			  permanently == YES ? @"YES" : @"NO");

	if (permanently == YES)
	{
		NSFileManager * fileManager = [NSFileManager defaultManager];
		[fileManager removeItemAtPath:path error:NULL];
	}
	else
	{
		NSInteger tag = 0;
		NSString * source = [path stringByDeletingLastPathComponent];
		NSArray * files = [NSArray arrayWithObject:[path lastPathComponent]];
		NSWorkspace * workspace = [NSWorkspace sharedWorkspace];
		[workspace performFileOperation:NSWorkspaceRecycleOperation
				   source:source
				   destination:nil
				   files:files
				   tag:&tag];
	}
}

@end
