#import "SlideShow.h"
#import "Utils.h"


@implementation SlideShow


static NSTimer * slideshowTimer = nil;


+(void)startSlideShow:(NSObject *)target callback:(NSString *)callback
{
	DEBUG_LOG(@"SlideShow - startSlideShow:%@ callback:%@",
			  [target className],
			  callback);

	if (slideshowTimer != nil)
		[SlideShow stopSlideShow];

	// get the slideshow user preferences
	NSUserDefaultsController * defaults = [NSUserDefaultsController sharedUserDefaultsController];
	BOOL loop = [[[defaults values] valueForKey:@"slideshowLoop"] boolValue];
	float interval = [[[defaults values] valueForKey:@"slideshowInterval"] floatValue];
	
	// launch the slideshow. I use a simple NSTimer with scheduledTimerWithTimeInterval.
	// This method creates the timer and automatically fire it after "interval" secondes.
	slideshowTimer = [NSTimer scheduledTimerWithTimeInterval:interval
							  target:target
							  selector:NSSelectorFromString(callback)
							  userInfo:[NSString stringWithFormat:@"%s", (loop ? "Y" : "N")]
							  repeats:YES];
	[slideshowTimer setFireDate:[NSDate dateWithTimeIntervalSinceNow:interval]];
}

+(void)stopSlideShow
{
	if (slideshowTimer != nil)
	{
		DEBUG_LOG(@"SlideShow - stopSlideShow");

		[slideshowTimer invalidate];
		slideshowTimer = nil;
	}
}

+(BOOL)isRunning
{
	return slideshowTimer == nil ? NO : YES;
}


@end
